@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,20 +13,20 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-
 import abc
-import typing
+from typing import Awaitable, Callable, Dict, Optional, Sequence, Union
+import packaging.version
 import pkg_resources
 
-from google import auth  # type: ignore
-from google.api_core import exceptions  # type: ignore
+import google.auth  # type: ignore
+import google.api_core  # type: ignore
+from google.api_core import exceptions as core_exceptions  # type: ignore
 from google.api_core import gapic_v1  # type: ignore
 from google.api_core import retry as retries  # type: ignore
-from google.auth import credentials  # type: ignore
+from google.auth import credentials as ga_credentials  # type: ignore
 
 from google.cloud.container_v1.types import cluster_service
-from google.protobuf import empty_pb2 as empty  # type: ignore
-
+from google.protobuf import empty_pb2  # type: ignore
 
 try:
     DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
@@ -36,27 +35,41 @@ try:
 except pkg_resources.DistributionNotFound:
     DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo()
 
+try:
+    # google.auth.__version__ was added in 1.26.0
+    _GOOGLE_AUTH_VERSION = google.auth.__version__
+except AttributeError:
+    try:  # try pkg_resources if it is available
+        _GOOGLE_AUTH_VERSION = pkg_resources.get_distribution("google-auth").version
+    except pkg_resources.DistributionNotFound:  # pragma: NO COVER
+        _GOOGLE_AUTH_VERSION = None
+
+_API_CORE_VERSION = google.api_core.__version__
+
 
 class ClusterManagerTransport(abc.ABC):
     """Abstract transport class for ClusterManager."""
 
     AUTH_SCOPES = ("https://www.googleapis.com/auth/cloud-platform",)
 
+    DEFAULT_HOST: str = "container.googleapis.com"
+
     def __init__(
         self,
         *,
-        host: str = "container.googleapis.com",
-        credentials: credentials.Credentials = None,
-        credentials_file: typing.Optional[str] = None,
-        scopes: typing.Optional[typing.Sequence[str]] = AUTH_SCOPES,
-        quota_project_id: typing.Optional[str] = None,
+        host: str = DEFAULT_HOST,
+        credentials: ga_credentials.Credentials = None,
+        credentials_file: Optional[str] = None,
+        scopes: Optional[Sequence[str]] = None,
+        quota_project_id: Optional[str] = None,
         client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
         **kwargs,
     ) -> None:
         """Instantiate the transport.
 
         Args:
-            host (Optional[str]): The hostname to connect to.
+            host (Optional[str]):
+                 The hostname to connect to.
             credentials (Optional[google.auth.credentials.Credentials]): The
                 authorization credentials to attach to requests. These
                 credentials identify the application to the service; if none
@@ -65,7 +78,7 @@ class ClusterManagerTransport(abc.ABC):
             credentials_file (Optional[str]): A file with credentials that can
                 be loaded with :func:`google.auth.load_credentials_from_file`.
                 This argument is mutually exclusive with credentials.
-            scope (Optional[Sequence[str]]): A list of scopes.
+            scopes (Optional[Sequence[str]]): A list of scopes.
             quota_project_id (Optional[str]): An optional project to use for billing
                 and quota.
             client_info (google.api_core.gapic_v1.client_info.ClientInfo):
@@ -79,29 +92,76 @@ class ClusterManagerTransport(abc.ABC):
             host += ":443"
         self._host = host
 
+        scopes_kwargs = self._get_scopes_kwargs(self._host, scopes)
+
         # Save the scopes.
         self._scopes = scopes or self.AUTH_SCOPES
 
         # If no credentials are provided, then determine the appropriate
         # defaults.
         if credentials and credentials_file:
-            raise exceptions.DuplicateCredentialArgs(
+            raise core_exceptions.DuplicateCredentialArgs(
                 "'credentials_file' and 'credentials' are mutually exclusive"
             )
 
         if credentials_file is not None:
-            credentials, _ = auth.load_credentials_from_file(
-                credentials_file, scopes=self._scopes, quota_project_id=quota_project_id
+            credentials, _ = google.auth.load_credentials_from_file(
+                credentials_file, **scopes_kwargs, quota_project_id=quota_project_id
             )
 
         elif credentials is None:
-            credentials, _ = auth.default(
-                scopes=self._scopes, quota_project_id=quota_project_id
+            credentials, _ = google.auth.default(
+                **scopes_kwargs, quota_project_id=quota_project_id
             )
 
         # Save the credentials.
         self._credentials = credentials
 
+    # TODO(busunkim): These two class methods are in the base transport
+    # to avoid duplicating code across the transport classes. These functions
+    # should be deleted once the minimum required versions of google-api-core
+    # and google-auth are increased.
+
+    # TODO: Remove this function once google-auth >= 1.25.0 is required
+    @classmethod
+    def _get_scopes_kwargs(
+        cls, host: str, scopes: Optional[Sequence[str]]
+    ) -> Dict[str, Optional[Sequence[str]]]:
+        """Returns scopes kwargs to pass to google-auth methods depending on the google-auth version"""
+
+        scopes_kwargs = {}
+
+        if _GOOGLE_AUTH_VERSION and (
+            packaging.version.parse(_GOOGLE_AUTH_VERSION)
+            >= packaging.version.parse("1.25.0")
+        ):
+            scopes_kwargs = {"scopes": scopes, "default_scopes": cls.AUTH_SCOPES}
+        else:
+            scopes_kwargs = {"scopes": scopes or cls.AUTH_SCOPES}
+
+        return scopes_kwargs
+
+    # TODO: Remove this function once google-api-core >= 1.26.0 is required
+    @classmethod
+    def _get_self_signed_jwt_kwargs(
+        cls, host: str, scopes: Optional[Sequence[str]]
+    ) -> Dict[str, Union[Optional[Sequence[str]], str]]:
+        """Returns kwargs to pass to grpc_helpers.create_channel depending on the google-api-core version"""
+
+        self_signed_jwt_kwargs: Dict[str, Union[Optional[Sequence[str]], str]] = {}
+
+        if _API_CORE_VERSION and (
+            packaging.version.parse(_API_CORE_VERSION)
+            >= packaging.version.parse("1.26.0")
+        ):
+            self_signed_jwt_kwargs["default_scopes"] = cls.AUTH_SCOPES
+            self_signed_jwt_kwargs["scopes"] = scopes
+            self_signed_jwt_kwargs["default_host"] = cls.DEFAULT_HOST
+        else:
+            self_signed_jwt_kwargs["scopes"] = scopes or cls.AUTH_SCOPES
+
+        return self_signed_jwt_kwargs
+
     def _prep_wrapped_messages(self, client_info):
         # Precompute the wrapped methods.
         self._wrapped_methods = {
@@ -112,7 +172,8 @@ class ClusterManagerTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded, exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.ServiceUnavailable,
                     ),
                     deadline=20.0,
                 ),
@@ -126,7 +187,8 @@ class ClusterManagerTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded, exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.ServiceUnavailable,
                     ),
                     deadline=20.0,
                 ),
@@ -174,7 +236,8 @@ class ClusterManagerTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded, exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.ServiceUnavailable,
                     ),
                     deadline=20.0,
                 ),
@@ -188,7 +251,8 @@ class ClusterManagerTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded, exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.ServiceUnavailable,
                     ),
                     deadline=20.0,
                 ),
@@ -202,7 +266,8 @@ class ClusterManagerTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded, exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.ServiceUnavailable,
                     ),
                     deadline=20.0,
                 ),
@@ -219,7 +284,8 @@ class ClusterManagerTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded, exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.ServiceUnavailable,
                     ),
                     deadline=20.0,
                 ),
@@ -236,7 +302,8 @@ class ClusterManagerTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded, exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.ServiceUnavailable,
                     ),
                     deadline=20.0,
                 ),
@@ -250,7 +317,8 @@ class ClusterManagerTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded, exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.ServiceUnavailable,
                     ),
                     deadline=20.0,
                 ),
@@ -267,7 +335,8 @@ class ClusterManagerTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded, exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.ServiceUnavailable,
                     ),
                     deadline=20.0,
                 ),
@@ -319,11 +388,11 @@ class ClusterManagerTransport(abc.ABC):
     @property
     def list_clusters(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.ListClustersRequest],
-        typing.Union[
+        Union[
             cluster_service.ListClustersResponse,
-            typing.Awaitable[cluster_service.ListClustersResponse],
+            Awaitable[cluster_service.ListClustersResponse],
         ],
     ]:
         raise NotImplementedError()
@@ -331,143 +400,119 @@ class ClusterManagerTransport(abc.ABC):
     @property
     def get_cluster(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.GetClusterRequest],
-        typing.Union[
-            cluster_service.Cluster, typing.Awaitable[cluster_service.Cluster]
-        ],
+        Union[cluster_service.Cluster, Awaitable[cluster_service.Cluster]],
     ]:
         raise NotImplementedError()
 
     @property
     def create_cluster(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.CreateClusterRequest],
-        typing.Union[
-            cluster_service.Operation, typing.Awaitable[cluster_service.Operation]
-        ],
+        Union[cluster_service.Operation, Awaitable[cluster_service.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def update_cluster(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.UpdateClusterRequest],
-        typing.Union[
-            cluster_service.Operation, typing.Awaitable[cluster_service.Operation]
-        ],
+        Union[cluster_service.Operation, Awaitable[cluster_service.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def update_node_pool(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.UpdateNodePoolRequest],
-        typing.Union[
-            cluster_service.Operation, typing.Awaitable[cluster_service.Operation]
-        ],
+        Union[cluster_service.Operation, Awaitable[cluster_service.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def set_node_pool_autoscaling(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.SetNodePoolAutoscalingRequest],
-        typing.Union[
-            cluster_service.Operation, typing.Awaitable[cluster_service.Operation]
-        ],
+        Union[cluster_service.Operation, Awaitable[cluster_service.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def set_logging_service(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.SetLoggingServiceRequest],
-        typing.Union[
-            cluster_service.Operation, typing.Awaitable[cluster_service.Operation]
-        ],
+        Union[cluster_service.Operation, Awaitable[cluster_service.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def set_monitoring_service(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.SetMonitoringServiceRequest],
-        typing.Union[
-            cluster_service.Operation, typing.Awaitable[cluster_service.Operation]
-        ],
+        Union[cluster_service.Operation, Awaitable[cluster_service.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def set_addons_config(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.SetAddonsConfigRequest],
-        typing.Union[
-            cluster_service.Operation, typing.Awaitable[cluster_service.Operation]
-        ],
+        Union[cluster_service.Operation, Awaitable[cluster_service.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def set_locations(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.SetLocationsRequest],
-        typing.Union[
-            cluster_service.Operation, typing.Awaitable[cluster_service.Operation]
-        ],
+        Union[cluster_service.Operation, Awaitable[cluster_service.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def update_master(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.UpdateMasterRequest],
-        typing.Union[
-            cluster_service.Operation, typing.Awaitable[cluster_service.Operation]
-        ],
+        Union[cluster_service.Operation, Awaitable[cluster_service.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def set_master_auth(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.SetMasterAuthRequest],
-        typing.Union[
-            cluster_service.Operation, typing.Awaitable[cluster_service.Operation]
-        ],
+        Union[cluster_service.Operation, Awaitable[cluster_service.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def delete_cluster(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.DeleteClusterRequest],
-        typing.Union[
-            cluster_service.Operation, typing.Awaitable[cluster_service.Operation]
-        ],
+        Union[cluster_service.Operation, Awaitable[cluster_service.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def list_operations(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.ListOperationsRequest],
-        typing.Union[
+        Union[
             cluster_service.ListOperationsResponse,
-            typing.Awaitable[cluster_service.ListOperationsResponse],
+            Awaitable[cluster_service.ListOperationsResponse],
         ],
     ]:
         raise NotImplementedError()
@@ -475,42 +520,38 @@ class ClusterManagerTransport(abc.ABC):
     @property
     def get_operation(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.GetOperationRequest],
-        typing.Union[
-            cluster_service.Operation, typing.Awaitable[cluster_service.Operation]
-        ],
+        Union[cluster_service.Operation, Awaitable[cluster_service.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def cancel_operation(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.CancelOperationRequest],
-        typing.Union[empty.Empty, typing.Awaitable[empty.Empty]],
+        Union[empty_pb2.Empty, Awaitable[empty_pb2.Empty]],
     ]:
         raise NotImplementedError()
 
     @property
     def get_server_config(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.GetServerConfigRequest],
-        typing.Union[
-            cluster_service.ServerConfig, typing.Awaitable[cluster_service.ServerConfig]
-        ],
+        Union[cluster_service.ServerConfig, Awaitable[cluster_service.ServerConfig]],
     ]:
         raise NotImplementedError()
 
     @property
     def get_json_web_keys(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.GetJSONWebKeysRequest],
-        typing.Union[
+        Union[
             cluster_service.GetJSONWebKeysResponse,
-            typing.Awaitable[cluster_service.GetJSONWebKeysResponse],
+            Awaitable[cluster_service.GetJSONWebKeysResponse],
         ],
     ]:
         raise NotImplementedError()
@@ -518,11 +559,11 @@ class ClusterManagerTransport(abc.ABC):
     @property
     def list_node_pools(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.ListNodePoolsRequest],
-        typing.Union[
+        Union[
             cluster_service.ListNodePoolsResponse,
-            typing.Awaitable[cluster_service.ListNodePoolsResponse],
+            Awaitable[cluster_service.ListNodePoolsResponse],
         ],
     ]:
         raise NotImplementedError()
@@ -530,143 +571,119 @@ class ClusterManagerTransport(abc.ABC):
     @property
     def get_node_pool(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.GetNodePoolRequest],
-        typing.Union[
-            cluster_service.NodePool, typing.Awaitable[cluster_service.NodePool]
-        ],
+        Union[cluster_service.NodePool, Awaitable[cluster_service.NodePool]],
     ]:
         raise NotImplementedError()
 
     @property
     def create_node_pool(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.CreateNodePoolRequest],
-        typing.Union[
-            cluster_service.Operation, typing.Awaitable[cluster_service.Operation]
-        ],
+        Union[cluster_service.Operation, Awaitable[cluster_service.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def delete_node_pool(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.DeleteNodePoolRequest],
-        typing.Union[
-            cluster_service.Operation, typing.Awaitable[cluster_service.Operation]
-        ],
+        Union[cluster_service.Operation, Awaitable[cluster_service.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def rollback_node_pool_upgrade(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.RollbackNodePoolUpgradeRequest],
-        typing.Union[
-            cluster_service.Operation, typing.Awaitable[cluster_service.Operation]
-        ],
+        Union[cluster_service.Operation, Awaitable[cluster_service.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def set_node_pool_management(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.SetNodePoolManagementRequest],
-        typing.Union[
-            cluster_service.Operation, typing.Awaitable[cluster_service.Operation]
-        ],
+        Union[cluster_service.Operation, Awaitable[cluster_service.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def set_labels(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.SetLabelsRequest],
-        typing.Union[
-            cluster_service.Operation, typing.Awaitable[cluster_service.Operation]
-        ],
+        Union[cluster_service.Operation, Awaitable[cluster_service.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def set_legacy_abac(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.SetLegacyAbacRequest],
-        typing.Union[
-            cluster_service.Operation, typing.Awaitable[cluster_service.Operation]
-        ],
+        Union[cluster_service.Operation, Awaitable[cluster_service.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def start_ip_rotation(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.StartIPRotationRequest],
-        typing.Union[
-            cluster_service.Operation, typing.Awaitable[cluster_service.Operation]
-        ],
+        Union[cluster_service.Operation, Awaitable[cluster_service.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def complete_ip_rotation(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.CompleteIPRotationRequest],
-        typing.Union[
-            cluster_service.Operation, typing.Awaitable[cluster_service.Operation]
-        ],
+        Union[cluster_service.Operation, Awaitable[cluster_service.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def set_node_pool_size(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.SetNodePoolSizeRequest],
-        typing.Union[
-            cluster_service.Operation, typing.Awaitable[cluster_service.Operation]
-        ],
+        Union[cluster_service.Operation, Awaitable[cluster_service.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def set_network_policy(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.SetNetworkPolicyRequest],
-        typing.Union[
-            cluster_service.Operation, typing.Awaitable[cluster_service.Operation]
-        ],
+        Union[cluster_service.Operation, Awaitable[cluster_service.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def set_maintenance_policy(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.SetMaintenancePolicyRequest],
-        typing.Union[
-            cluster_service.Operation, typing.Awaitable[cluster_service.Operation]
-        ],
+        Union[cluster_service.Operation, Awaitable[cluster_service.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def list_usable_subnetworks(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [cluster_service.ListUsableSubnetworksRequest],
-        typing.Union[
+        Union[
             cluster_service.ListUsableSubnetworksResponse,
-            typing.Awaitable[cluster_service.ListUsableSubnetworksResponse],
+            Awaitable[cluster_service.ListUsableSubnetworksResponse],
         ],
     ]:
         raise NotImplementedError()
