@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,11 +15,13 @@
 #
 
 from .services.cluster_manager import ClusterManagerClient
+from .services.cluster_manager import ClusterManagerAsyncClient
+
 from .types.cluster_service import AcceleratorConfig
 from .types.cluster_service import AddonsConfig
 from .types.cluster_service import AuthenticatorGroupsConfig
-from .types.cluster_service import AutoUpgradeOptions
 from .types.cluster_service import AutoprovisioningNodePoolDefaults
+from .types.cluster_service import AutoUpgradeOptions
 from .types.cluster_service import BinaryAuthorization
 from .types.cluster_service import CancelOperationRequest
 from .types.cluster_service import ClientCertificateConfig
@@ -36,7 +37,6 @@ from .types.cluster_service import CreateClusterRequest
 from .types.cluster_service import CreateNodePoolRequest
 from .types.cluster_service import DailyMaintenanceWindow
 from .types.cluster_service import DatabaseEncryption
-from .types.cluster_service import DatapathProvider
 from .types.cluster_service import DefaultSnatStatus
 from .types.cluster_service import DeleteClusterRequest
 from .types.cluster_service import DeleteNodePoolRequest
@@ -53,8 +53,8 @@ from .types.cluster_service import GetOperationRequest
 from .types.cluster_service import GetServerConfigRequest
 from .types.cluster_service import HorizontalPodAutoscaling
 from .types.cluster_service import HttpLoadBalancing
-from .types.cluster_service import IPAllocationPolicy
 from .types.cluster_service import IntraNodeVisibilityConfig
+from .types.cluster_service import IPAllocationPolicy
 from .types.cluster_service import IstioConfig
 from .types.cluster_service import Jwk
 from .types.cluster_service import KalmConfig
@@ -123,15 +123,16 @@ from .types.cluster_service import UpdateClusterRequest
 from .types.cluster_service import UpdateMasterRequest
 from .types.cluster_service import UpdateNodePoolRequest
 from .types.cluster_service import UpgradeEvent
-from .types.cluster_service import UpgradeResourceType
 from .types.cluster_service import UsableSubnetwork
 from .types.cluster_service import UsableSubnetworkSecondaryRange
 from .types.cluster_service import VerticalPodAutoscaling
 from .types.cluster_service import WorkloadIdentityConfig
 from .types.cluster_service import WorkloadMetadataConfig
-
+from .types.cluster_service import DatapathProvider
+from .types.cluster_service import UpgradeResourceType
 
 __all__ = (
+    "ClusterManagerAsyncClient",
     "AcceleratorConfig",
     "AddonsConfig",
     "AuthenticatorGroupsConfig",
@@ -143,6 +144,7 @@ __all__ = (
     "CloudRunConfig",
     "Cluster",
     "ClusterAutoscaling",
+    "ClusterManagerClient",
     "ClusterTelemetry",
     "ClusterUpdate",
     "CompleteIPRotationRequest",
@@ -245,5 +247,4 @@ __all__ = (
     "VerticalPodAutoscaling",
     "WorkloadIdentityConfig",
     "WorkloadMetadataConfig",
-    "ClusterManagerClient",
 )
