@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,21 +13,19 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-
 import warnings
-from typing import Callable, Dict, Optional, Sequence, Tuple
+from typing import Callable, Dict, Optional, Sequence, Tuple, Union
 
 from google.api_core import grpc_helpers  # type: ignore
 from google.api_core import gapic_v1  # type: ignore
-from google import auth  # type: ignore
-from google.auth import credentials  # type: ignore
+import google.auth  # type: ignore
+from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
 
 import grpc  # type: ignore
 
 from google.cloud.container_v1beta1.types import cluster_service
-from google.protobuf import empty_pb2 as empty  # type: ignore
-
+from google.protobuf import empty_pb2  # type: ignore
 from .base import ClusterManagerTransport, DEFAULT_CLIENT_INFO
 
 
@@ -51,7 +48,7 @@ class ClusterManagerGrpcTransport(ClusterManagerTransport):
         self,
         *,
         host: str = "container.googleapis.com",
-        credentials: credentials.Credentials = None,
+        credentials: ga_credentials.Credentials = None,
         credentials_file: str = None,
         scopes: Sequence[str] = None,
         channel: grpc.Channel = None,
@@ -65,7 +62,8 @@ class ClusterManagerGrpcTransport(ClusterManagerTransport):
         """Instantiate the transport.
 
         Args:
-            host (Optional[str]): The hostname to connect to.
+            host (Optional[str]):
+                 The hostname to connect to.
             credentials (Optional[google.auth.credentials.Credentials]): The
                 authorization credentials to attach to requests. These
                 credentials identify the application to the service; if none
@@ -175,7 +173,7 @@ class ClusterManagerGrpcTransport(ClusterManagerTransport):
     def create_channel(
         cls,
         host: str = "container.googleapis.com",
-        credentials: credentials.Credentials = None,
+        credentials: ga_credentials.Credentials = None,
         credentials_file: str = None,
         scopes: Optional[Sequence[str]] = None,
         quota_project_id: Optional[str] = None,
@@ -206,13 +204,15 @@ class ClusterManagerGrpcTransport(ClusterManagerTransport):
             google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
               and ``credentials_file`` are passed.
         """
-        scopes = scopes or cls.AUTH_SCOPES
+
+        self_signed_jwt_kwargs = cls._get_self_signed_jwt_kwargs(host, scopes)
+
         return grpc_helpers.create_channel(
             host,
             credentials=credentials,
             credentials_file=credentials_file,
-            scopes=scopes,
             quota_project_id=quota_project_id,
+            **self_signed_jwt_kwargs,
             **kwargs,
         )
 
@@ -655,7 +655,7 @@ class ClusterManagerGrpcTransport(ClusterManagerTransport):
     @property
     def cancel_operation(
         self,
-    ) -> Callable[[cluster_service.CancelOperationRequest], empty.Empty]:
+    ) -> Callable[[cluster_service.CancelOperationRequest], empty_pb2.Empty]:
         r"""Return a callable for the cancel operation method over gRPC.
 
         Cancels the specified operation.
@@ -674,7 +674,7 @@ class ClusterManagerGrpcTransport(ClusterManagerTransport):
             self._stubs["cancel_operation"] = self.grpc_channel.unary_unary(
                 "/google.container.v1beta1.ClusterManager/CancelOperation",
                 request_serializer=cluster_service.CancelOperationRequest.serialize,
-                response_deserializer=empty.Empty.FromString,
+                response_deserializer=empty_pb2.Empty.FromString,
             )
         return self._stubs["cancel_operation"]
 
