@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,22 +13,20 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-
 import warnings
-from typing import Awaitable, Callable, Dict, Optional, Sequence, Tuple
+from typing import Awaitable, Callable, Dict, Optional, Sequence, Tuple, Union
 
 from google.api_core import gapic_v1  # type: ignore
 from google.api_core import grpc_helpers_async  # type: ignore
-from google import auth  # type: ignore
-from google.auth import credentials  # type: ignore
+from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
+import packaging.version
 
 import grpc  # type: ignore
 from grpc.experimental import aio  # type: ignore
 
 from google.cloud.container_v1beta1.types import cluster_service
-from google.protobuf import empty_pb2 as empty  # type: ignore
-
+from google.protobuf import empty_pb2  # type: ignore
 from .base import ClusterManagerTransport, DEFAULT_CLIENT_INFO
 from .grpc import ClusterManagerGrpcTransport
 
@@ -54,7 +51,7 @@ class ClusterManagerGrpcAsyncIOTransport(ClusterManagerTransport):
     def create_channel(
         cls,
         host: str = "container.googleapis.com",
-        credentials: credentials.Credentials = None,
+        credentials: ga_credentials.Credentials = None,
         credentials_file: Optional[str] = None,
         scopes: Optional[Sequence[str]] = None,
         quota_project_id: Optional[str] = None,
@@ -81,13 +78,15 @@ class ClusterManagerGrpcAsyncIOTransport(ClusterManagerTransport):
         Returns:
             aio.Channel: A gRPC AsyncIO channel object.
         """
-        scopes = scopes or cls.AUTH_SCOPES
+
+        self_signed_jwt_kwargs = cls._get_self_signed_jwt_kwargs(host, scopes)
+
         return grpc_helpers_async.create_channel(
             host,
             credentials=credentials,
             credentials_file=credentials_file,
-            scopes=scopes,
             quota_project_id=quota_project_id,
+            **self_signed_jwt_kwargs,
             **kwargs,
         )
 
@@ -95,7 +94,7 @@ class ClusterManagerGrpcAsyncIOTransport(ClusterManagerTransport):
         self,
         *,
         host: str = "container.googleapis.com",
-        credentials: credentials.Credentials = None,
+        credentials: ga_credentials.Credentials = None,
         credentials_file: Optional[str] = None,
         scopes: Optional[Sequence[str]] = None,
         channel: aio.Channel = None,
@@ -109,7 +108,8 @@ class ClusterManagerGrpcAsyncIOTransport(ClusterManagerTransport):
         """Instantiate the transport.
 
         Args:
-            host (Optional[str]): The hostname to connect to.
+            host (Optional[str]):
+                 The hostname to connect to.
             credentials (Optional[google.auth.credentials.Credentials]): The
                 authorization credentials to attach to requests. These
                 credentials identify the application to the service; if none
@@ -167,7 +167,6 @@ class ClusterManagerGrpcAsyncIOTransport(ClusterManagerTransport):
             # If a channel was explicitly provided, set it.
             self._grpc_channel = channel
             self._ssl_channel_credentials = None
-
         else:
             if api_mtls_endpoint:
                 host = api_mtls_endpoint
@@ -683,7 +682,7 @@ class ClusterManagerGrpcAsyncIOTransport(ClusterManagerTransport):
     @property
     def cancel_operation(
         self,
-    ) -> Callable[[cluster_service.CancelOperationRequest], Awaitable[empty.Empty]]:
+    ) -> Callable[[cluster_service.CancelOperationRequest], Awaitable[empty_pb2.Empty]]:
         r"""Return a callable for the cancel operation method over gRPC.
 
         Cancels the specified operation.
@@ -702,7 +701,7 @@ class ClusterManagerGrpcAsyncIOTransport(ClusterManagerTransport):
             self._stubs["cancel_operation"] = self.grpc_channel.unary_unary(
                 "/google.container.v1beta1.ClusterManager/CancelOperation",
                 request_serializer=cluster_service.CancelOperationRequest.serialize,
-                response_deserializer=empty.Empty.FromString,
+                response_deserializer=empty_pb2.Empty.FromString,
             )
         return self._stubs["cancel_operation"]
 
